'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var MiniCssExtractPlugin = _interopDefault(require('mini-css-extract-plugin'));
var withTM = _interopDefault(require('next-transpile-modules'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

module.exports = function withHydrationInitializer(scriptsByPath) {
    return function withHydration(receivedConfig) {
        if (receivedConfig === void 0) { receivedConfig = {}; }
        // Since we use the `DefinePlugin` to inject the config into `ClientScript`,
        // we need to make sure that this package gets transpiled. Probably it would
        // also be possible to configure the loaders/plugin directly here without
        // using this dependency.
        return withTM(['next-client-script'])(__assign(__assign({}, receivedConfig), { webpack: function (config, options) {
                var _a;
                var buildId = options.buildId, dev = options.dev, isServer = options.isServer, nextWebpack = options.webpack;
                var clientEntries = {};
                var clientScriptsByPath = {};
                function encodePath(scriptPath) {
                    return scriptPath
                        .replace(/\W/g, '-')
                        .replace(/-(js|jsx|ts|tsx)$/, '');
                }
                Object.entries(scriptsByPath).forEach(function (_a) {
                    var pagePath = _a[0], scriptPath = _a[1];
                    var publicBasePath = '/static/client/';
                    var publicPath = publicBasePath + encodePath(scriptPath);
                    if (!dev)
                        publicPath += "-" + buildId;
                    else
                        publicPath += '.js';
                    clientScriptsByPath[pagePath] = '/_next' + publicPath;
                    if (!dev)
                        clientScriptsByPath[pagePath] += '.js';
                    var entryPointName = publicPath.substring(1);
                    clientEntries[entryPointName] = scriptPath;
                });
                var CLIENT_SCRIPTS_BY_PATH = JSON.stringify(clientScriptsByPath);
                config = __assign(__assign({}, config), { plugins: (_a = config.plugins) === null || _a === void 0 ? void 0 : _a.concat(new nextWebpack.DefinePlugin({
                        CLIENT_SCRIPTS_BY_PATH: CLIENT_SCRIPTS_BY_PATH
                    })) });
                if (isServer) {
                    return config;
                }
                if (dev) {
                    var originalEntry_1 = config.entry;
                    config.entry = function () {
                        return originalEntry_1().then(function (entry) { return (__assign(__assign({}, entry), clientEntries)); });
                    };
                    // Overload the Webpack config if it was already overloaded
                    if (typeof receivedConfig.webpack === 'function') {
                        return receivedConfig.webpack(config, options);
                    }
                    return config;
                }
                else {
                    var buildIdPath = path.resolve(process.cwd(), './.next/BUILD_ID');
                    // TODO: create folder if it doesn't exist.
                    // TODO: check what happens when the file already exists
                    fs.writeFileSync(buildIdPath, buildId);
                    var clientConfig = __assign(__assign({}, config), { entry: clientEntries, output: {
                            path: path.resolve(process.cwd(), './.next/')
                        }, optimization: __assign(__assign({}, config.optimization), { 
                            // Output only a single JavaScript asset that
                            // contains all necessary client code.
                            runtimeChunk: undefined }), plugins: (config.plugins || [])
                            .filter(function (plugin) {
                            return ![
                                // The manifest from the regular build should be used
                                'ReactLoadablePlugin',
                                'BuildManifestPlugin',
                                // Creates unnecessary sub folders
                                'NextMiniCssExtractPlugin'
                            ].includes(plugin.constructor.name);
                        })
                            .concat(
                        // Ideally we'd ignore the CSS assets, but if we don't use
                        // this plugin, the JS assets won't get emitted as well.
                        new MiniCssExtractPlugin()) });
                    var compiler = nextWebpack(clientConfig);
                    compiler.run(function (error, stats) {
                        var errorMessage;
                        if (error) {
                            errorMessage = error.message;
                        }
                        if (stats.compilation.errors.length > 0) {
                            errorMessage = stats.compilation.errors
                                .map(function (error) { return JSON.stringify(error); })
                                .join('\n\n');
                        }
                        if (errorMessage) {
                            console.error(errorMessage);
                            process.exit(1);
                        }
                    });
                    return config;
                }
            } }));
    };
};
//# sourceMappingURL=withClientScripts.js.map
