'use strict';

function initWidgets(widgets) {
    function runInitialization() {
        widgets.forEach(function (widget) {
            var nodes = document.querySelectorAll(widget.selector);
            for (var index = 0; index < nodes.length; index++) {
                var node = nodes[index];
                if (node instanceof HTMLElement) {
                    var data = void 0;
                    if (node.children) {
                        var dataChild = Array.from(node.children).find(function (child) {
                            return child instanceof HTMLElement &&
                                child.dataset.widgetData === 'true';
                        });
                        if (dataChild === null || dataChild === void 0 ? void 0 : dataChild.textContent) {
                            try {
                                data = JSON.parse(dataChild.textContent);
                            }
                            catch (error) {
                                console.error('Data unparseable: ' + dataChild.textContent);
                            }
                        }
                    }
                    widget(node, data);
                }
            }
        });
    }
    if (document.readyState === 'complete') {
        runInitialization();
    }
    else {
        window.addEventListener('load', runInitialization);
    }
}

module.exports = initWidgets;
//# sourceMappingURL=initWidgets.js.map
